---
title: "Universal Feed API"
order: 8
---

Universal Feeds can be queried using the [Common Packages API](/docs/proget/reference-api/proget-api-packages). There is also a special Universal Feed API, which was developed before the Common Packages API. While the Universal Feed API is still functional, we recommend using the Common Packages API if a required  endpoint is available.

## Universal Feed Endpoints

These endpoints are part of the original Universal Feed API and offer additional functionality. The API consists of a few different URL endpoints, accessible over HTTP/S. If configured, they can be secured using Integrated Windows Authentication or Basic Authentication using whatever granular feed- or system-level privileges needed. All endpoints are prefixed with `/upack/«feed-name»`:

* [Import Universal Package](/docs/proget/reference-api/universal-feed/import) - imports a package from specified source
* [Upload Universal Package](/docs/proget/reference-api/universal-feed/upload) - uploads a specified package
* [Download Virtual Package](/docs/proget/reference-api/universal-feed/download-vpack) - downloads a specified virtual package
* [Download Universal Package File](/docs/proget/reference-api/universal-feed/download-file) - downloads a specified file within a package

Note that the endpoint prefix itself is not a valid API endpoint, and may return a `404` error, or simply redirect you to this page.

Currently, the API only returns results in JSON format with a standard, `200` (success) status code unless there's an error; future versions (if anyone requests it) may add additional formats, such as XML, which would be specified using a `Content-Type` request headers and/or an alternate querystring parameter.

All searching and matching is case insensitive. This will most certainly never change, as it's either a mistake or a bad practice to have different packages named `SomeThing` and `someThing`.

## Deprecated Universal Feed Endpoints:

These endpoints are deprecated, and we recommend using the appropriate endpoints in the [Common Packages API](/docs/proget/reference-api/proget-api-packages) instead.

* [List Universal Packages]() - lists specified packages
* [Delete Universal Package]() - deletes a specified package
* [Download Universal Package]() - downloads a specified package
* [List Universal Package Versions]() - describes versions of specified packages

## Authenticating to API Endpoints { #authentication }

The following is a summary of access types and their corresponding requirements for various API keys types and endpoints within this API.

| Access Type | Requirements |
| --- | --- |
| System API Keys | `Use/Manage Feeds`  
| Feed API Keys | `View/Download` is required for all endpoints<br/>`Add/Repackage` is required for uploading/importing endpoints<br/>`Overwrite/Delete` is required for deletion endpoints
| Personal API Key | `Feeds_ViewFeed` is required for all endpoints<br/>`Feeds_AddPackage` is required for uploading/importing endpoints<br/>`Feeds_DeletePackage` is required for deletion endpoints
| No API Key | anonymous or authenticated user must have at least `Feeds_ViewFeed`

To specify an API Key, use the request header (`X-ApiKey`), querystring (`key`), or `api:«api-key»` as the username. See [API Key Usage](/docs/proget/reference-api/proget-apikeys#using-api-keys) to learn more.

:::(Info) (🚀 Quick Example: Authenticating with curl)
For example, to authenticate with the API key `abc12345` to the "Import Universal Package" endpoint, you could specify the API key as follows:
````
curl -X GET --header "X-ApiKey: abc12345" "https://proget.corp.local/upack/MyUniversalFeed/import"
````
:::

## Data Specifications { #data-specifications }

### PackageData Object Attributes { #package-data }

`PackageData` is a set of key/value pairs that correspond to the metadata fields in a Universal Package. It's used as output data for the [List Packages]()
 Endpoint as a JSON-formatted object.

| Property | Format| Notes|
|---|---|---|
| `group` | string | the group of the package|
| `name` | string | the name of the package |
| `latestVersion` | float | the latest version of the package |
| `title` | string | the title of the package |
| `description` | string | the description of the package |
| `downloads` | int | number of downloads performed |
| `isLocal` | bool | indicates if the package is hosted locally |
| `isCached` | bool | indicates if the package is cached |
| `versions` | array | an array of existing version numbers |

#### JSON Object

```json
[
  {
    "group": "MyGroup",
    "name": "MyUniversalPackage",
    "latestVersion": "2.0.4",
    "title": "Test Package",
    "description": "Test Package for API Documentation",
    "downloads": "5",
    "isLocal": true,
    "isCached": false,
    "versions": ["2.0.4", "1.2.3", "1.0.0"]
  },
  { ... } // other packages
]
```

### VersionData Object Attributes { #version-data }

`VersionData` is a set of key/value pairs that correspond to the metadata fields in a Universal Package version. It's used as output data for the [List Versions]()
 Endpoint as a JSON-formatted object.

| Property | Format| Notes|
|---|---|---|
| `isLocal` | bool | indicates if the package is hosted locally |
| `isCached` | bool | indicates if the package is cached |
| `isVirtual` | bool | indicates if the package is a virtual package |
| `sha1` | string | `hash` of package |
| `title` | string | the title of the package |
| `description` | string | the description of the package |
| `group` | string | the group of the package |
| `name` | string | the name of the package |
| `version` | float | the version of the package |
| `downloads` | int | number of downloads performed |
| `published` | datetime | timestamp when package was published to the feed |
| `size` | int | the size of the package in bytes |	
| `fileList` | array | an array of [FileData](#file-data) objects |	

#### JSON Object

```json
{
  "isLocal": true,
  "isCached": false,
  "isVirtual": false,
  "sha1": "00c1624a6bc2fbed3a8b49c9c90fa4f2b4a1241e",
  "title": "Test Package",
  "description": "Test Package for API Documentation",
  "group": "MyGroup",
  "name": "MyUniversalPackage",
  "version": "1.0.0",
  "downloads": 6,
  "published": "2023-12-11T02:13:47.86Z",
  "size": 1435884,
  "fileList": [ FileData objects ]
}
```

### FileData Object Attributes { #file-data }

`FileData` is a subset of attributes on the [VersionData](#version-data) Object that corresponds to fields of a file within a universal package. It's used as output data for the [List Versions]()
 Endpoint as a JSON-formatted object.

| Property | Format| Notes|
|---|---|---|
| `name` | string | the name of the package |
| `size` | int | the size of the package in bytes |	
| `date` | datetime | timestamp when file was added to package |	

#### JSON Object

```json
{
    "name": "example.bin",
    "size": 1543944,
    "date": "2023-12-08T10:25:50Z"
}
```